# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# by Troy Dawson <dawson@fnal.gov>
#  borrowing code from 
#    versionlock.py by Panu Matilainen <pmatilai@laiskiainen.org>
#    allowdowngrade.py by Panu Matilainen <pmatilai@laiskiainen.org>
#

try:
    set = set
except:
    from sets import Set as set

from yum.plugins import PluginYumExit
from yum.plugins import TYPE_CORE
from rpmUtils.miscutils import splitFilename, compareEVR
from rpm import RPMPROB_FILTER_OLDPACKAGE
import urlgrabber
import urlgrabber.grabber
import re
import rpm

from yum.plugins import TYPE_CORE

requires_api_version = '2.1'
plugin_type = (TYPE_CORE,)

def vl_search(conduit, name):
    """ Search for packages with a particular name or string. """
    return conduit._base.pkgSack.searchAll(name)
    

def exclude_hook(conduit):
    conduit.info(2, 'Beginning versionfix')
    conduit.info(4, '  Reading in fixlist')
    fixlist = []
    fixdict = {}
    fixlistname = []
    location = conduit.confString('main', 'fixlist')
    if not location:
        raise PluginYumExit('Fixlist not set')
    try:
        llfile = urlgrabber.urlopen(location)
        for line in llfile.readlines():
            if line.startswith('#') or line.strip() == '':
                continue
            wholeline = line.rstrip()
	    linesplit = line.rstrip().split('=')
	    conduit.info(4, 'wholeline: %s' % wholeline)
	    conduit.info(6, 'first: %s' % linesplit)
	    fixdict[linesplit[0]] = [linesplit[1]]
            (n, v, r, e, a) = splitFilename("%s" % linesplit[0])
            fixlist.append(linesplit[0])
            fixlistname.append(n)
        llfile.close()
    except urlgrabber.grabber.URLGrabError, e:
        raise PluginYumExit('Unable to read versionfix configuration: %s' % e)

    for hdr in conduit.getRpmDB().getHdrList():
        if hdr[rpm.RPMTAG_NAME] in fixlistname:
            conduit.info(4, '  RPM name in list: %s' %(hdr[rpm.RPMTAG_NAME]))
            for pkg in fixdict.keys():
		conduit.info(6, '    Checking on %s ' % pkg)
             	(n, v, r, e, a) = splitFilename("%s" % pkg)
            	palist = (hdr[rpm.RPMTAG_NAME], hdr[rpm.RPMTAG_VERSION], hdr[rpm.RPMTAG_RELEASE], hdr[rpm.RPMTAG_ARCH])
            	pklist = (n, v, r, a)
		if palist == pklist:
			  conduit.info(4, '      WE HAVE A MATCH: %s' %(hdr[rpm.RPMTAG_NAME]))
			  newpackage = fixdict.get(pkg)[0]
			  conduit.info(4, '        oldpackage: %s' %(pkg))
			  conduit.info(4, '        newpackage: %s' % newpackage)
			  (nn, nv, nr, ne, na) = splitFilename("%s" % newpackage)
			  for pkg in vl_search(conduit, nn):
			    palist = (pkg.name, pkg.version, pkg.release, pkg.epoch, pkg.arch)
			    pklist = (nn, nv, nr, ne, na)
			    conduit.info(6, 'palist %s pklist %s ' %(palist, pklist))
			    if palist == pklist:
			      conduit.info(4, '          We found package %s:' % pkg)
			      conduit.info(2, 'Package %s needs to be updated to %s' %(hdr[rpm.RPMTAG_NAME], pkg))
			      tsInfo = conduit.getTsInfo()
			      tsInfo.probFilterFlags.append(RPMPROB_FILTER_OLDPACKAGE)
			      tsInfo.addInstall(pkg)
    conduit.info(2, 'Ending versionfix')
